@@ -2,7 +2,7 @@
 
 pub use self::Term::*;
 pub use self::Notation::*;
-use self::Error::*;
+use self::TermError::*;
 use std::fmt;
 use std::borrow::Cow;
 use std::char::from_u32;
@@ -16,20 +16,25 @@ pub const LAMBDA: char = '\\';
 #[cfg(not(feature = "backslash_lambda"))]
 pub const LAMBDA: char = 'λ';
 
+/// An undefined term that can be used as a value returned by invalid/inapplicable operations, e.g.
+/// obtaining an element of an empty list. Since this implementation uses De Bruijn indices greater
+/// than zero, `Var(0)` will not occur naturally. It is displayed as `undefined`.
+pub const UD: Term = Var(0);
+
 /// The notation used for parsing and displaying purposes.
 ///
-/// # Example
+/// # Examples
 /// ```
-/// use lambda_calculus::combinators::s;
+/// use lambda_calculus::combinators::S;
 ///
-/// assert_eq!(&format!(  "{}", s()), "λa.λb.λc.a c (b c)"); // Classic notation
-/// assert_eq!(&format!("{:?}", s()), "λλλ31(21)");          // DeBruijn index notation
+/// assert_eq!(&format!(  "{}", S()), "λa.λb.λc.a c (b c)"); // Classic notation
+/// assert_eq!(&format!("{:?}", S()), "λλλ31(21)");          // DeBruijn index notation
 /// ```
 #[derive(Debug, PartialEq, Clone, Copy)]
 pub enum Notation {
-    /// classic lambda calculus notation; the default `fmt::Display` mode
+    /// classic lambda calculus notation; used by `fmt::Display`
     Classic,
-    /// De Bruijn indices; the `fmt::Debug` display mode
+    /// De Bruijn indices; used by `fmt::Debug`
     DeBruijn
 }
 
@@ -45,262 +50,276 @@ pub enum Term {
     App(Box<Term>, Box<Term>)
 }
 
-/// An error that can be returned when an inapplicable function is applied to a term.
+/// An error that can be returned when an inapplicable function is applied to a `Term`.
 #[derive(Debug, PartialEq)]
-pub enum Error {
+pub enum TermError {
     /// the term is not a variable
-    NotAVar,
+    NotVar,
     /// the term is not an abstraction
-    NotAnAbs,
+    NotAbs,
     /// the term is not an application
-    NotAnApp,
-    /// the term is not a Church number
-    NotANum,
-    /// the term is not a Church pair
-    NotAPair,
-    /// the term is not a Church list
-    NotAList
+    NotApp,
 }
 
 impl Term {
-    /// Applies `self` to another term without substitution or reduction.
+    /// Returns a variable's De Bruijn index, consuming it in the process.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
-    ///
-    /// assert_eq!(Var(1).app(Var(2)), App(Box::new(Var(1)), Box::new(Var(2))));
-    /// ```
-    pub fn app(self, argument: Term) -> Term { App(Box::new(self), Box::new(argument)) }
-
-    /// Consumes a lambda variable and returns its De Bruijn index.
-    ///
-    /// # Example
-    /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(Var(1).unvar(), Ok(1));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not a `Var`iable.
-    pub fn unvar(self) -> Result<usize, Error> {
-        if let Var(n) = self { Ok(n) } else { Err(NotAVar) }
+    /// Returns a `TermError` if `self` is not a `Var`iable.
+    pub fn unvar(self) -> Result<usize, TermError> {
+        if let Var(n) = self { Ok(n) } else { Err(NotVar) }
     }
 
-    /// Returns a reference to a variable's index.
+    /// Returns a reference to a variable's De Bruijn index.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(Var(1).unvar_ref(), Ok(&1));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not a `Var`iable.
-    pub fn unvar_ref(&self) -> Result<&usize, Error> {
-        if let Var(ref n) = *self { Ok(n) } else { Err(NotAVar) }
+    /// Returns a `TermError` if `self` is not a `Var`iable.
+    pub fn unvar_ref(&self) -> Result<&usize, TermError> {
+        if let Var(ref n) = *self { Ok(n) } else { Err(NotVar) }
     }
 
-    /// Returns a mutable reference to a variable's index.
+    /// Returns a mutable reference to a variable's De Bruijn index.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(Var(1).unvar_mut(), Ok(&mut 1));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not a `Var`iable.
-    pub fn unvar_mut(&mut self) -> Result<&mut usize, Error> {
-        if let Var(ref mut n) = *self { Ok(n) } else { Err(NotAVar) }
+    /// Returns a `TermError` if `self` is not a `Var`iable.
+    pub fn unvar_mut(&mut self) -> Result<&mut usize, TermError> {
+        if let Var(ref mut n) = *self { Ok(n) } else { Err(NotVar) }
     }
 
-    /// Consumes an abstraction and returns its underlying term.
+    /// Returns an abstraction's underlying term, consuming it in the process.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(abs(Var(1)).unabs(), Ok(Var(1)));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `Abs`traction.
-    pub fn unabs(self) -> Result<Term, Error> {
-        if let Abs(x) = self { Ok(*x) } else { Err(NotAnAbs) }
+    /// Returns a `TermError` if `self` is not an `Abs`traction.
+    pub fn unabs(self) -> Result<Term, TermError> {
+        if let Abs(x) = self { Ok(*x) } else { Err(NotAbs) }
     }
 
     /// Returns a reference to an abstraction's underlying term.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(abs(Var(1)).unabs_ref(), Ok(&Var(1)));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `Abs`traction.
-    pub fn unabs_ref(&self) -> Result<&Term, Error> {
-        if let Abs(ref x) = *self { Ok(x) } else { Err(NotAnAbs) }
+    /// Returns a `TermError` if `self` is not an `Abs`traction.
+    pub fn unabs_ref(&self) -> Result<&Term, TermError> {
+        if let Abs(ref x) = *self { Ok(x) } else { Err(NotAbs) }
     }
 
     /// Returns a mutable reference to an abstraction's underlying term.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(abs(Var(1)).unabs_mut(), Ok(&mut Var(1)));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `Abs`traction.
-    pub fn unabs_mut(&mut self) -> Result<&mut Term, Error> {
-        if let Abs(ref mut x) = *self { Ok(x) } else { Err(NotAnAbs) }
+    /// Returns a `TermError` if `self` is not an `Abs`traction.
+    pub fn unabs_mut(&mut self) -> Result<&mut Term, TermError> {
+        if let Abs(ref mut x) = *self { Ok(x) } else { Err(NotAbs) }
     }
 
-    /// Consumes an application and returns a pair containing its underlying terms.
+    /// Returns a pair containing an application's underlying terms, consuming it in the process.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(app(Var(1), Var(2)).unapp(), Ok((Var(1), Var(2))));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `App`lication.
-    pub fn unapp(self) -> Result<(Term, Term), Error> {
-        if let App(lhs, rhs) = self { Ok((*lhs, *rhs)) } else { Err(NotAnApp) }
+    /// Returns a `TermError` if `self` is not an `App`lication.
+    pub fn unapp(self) -> Result<(Term, Term), TermError> {
+        if let App(lhs, rhs) = self { Ok((*lhs, *rhs)) } else { Err(NotApp) }
     }
 
     /// Returns a pair containing references to an application's underlying terms.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(app(Var(1), Var(2)).unapp_ref(), Ok((&Var(1), &Var(2))));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `App`lication.
-    pub fn unapp_ref(&self) -> Result<(&Term, &Term), Error> {
-        if let App(ref lhs, ref rhs) = *self { Ok((lhs, rhs)) } else { Err(NotAnApp) }
+    /// Returns a `TermError` if `self` is not an `App`lication.
+    pub fn unapp_ref(&self) -> Result<(&Term, &Term), TermError> {
+        if let App(ref lhs, ref rhs) = *self { Ok((lhs, rhs)) } else { Err(NotApp) }
     }
 
     /// Returns a pair containing mutable references to an application's underlying terms.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(app(Var(1), Var(2)).unapp_mut(), Ok((&mut Var(1), &mut Var(2))));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `App`lication.
-    pub fn unapp_mut(&mut self) -> Result<(&mut Term, &mut Term), Error> {
-        if let App(ref mut lhs, ref mut rhs) = *self { Ok((lhs, rhs)) } else { Err(NotAnApp) }
+    /// Returns a `TermError` if `self` is not an `App`lication.
+    pub fn unapp_mut(&mut self) -> Result<(&mut Term, &mut Term), TermError> {
+        if let App(ref mut lhs, ref mut rhs) = *self { Ok((lhs, rhs)) } else { Err(NotApp) }
     }
 
     /// Returns the left-hand side term of an application. Consumes `self`.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
-    /// assert_eq!(Var(1).app(Var(2)).lhs(), Ok(Var(1)));
+    /// assert_eq!(app(Var(1), Var(2)).lhs(), Ok(Var(1)));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `App`lication.
-    pub fn lhs(self) -> Result<Term, Error> {
-        if let Ok((lhs, _)) = self.unapp() { Ok(lhs) } else { Err(NotAnApp) }
+    /// Returns a `TermError` if `self` is not an `App`lication.
+    pub fn lhs(self) -> Result<Term, TermError> {
+        if let Ok((lhs, _)) = self.unapp() { Ok(lhs) } else { Err(NotApp) }
     }
 
     /// Returns a reference to the left-hand side term of an application.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(app(Var(1), Var(2)).lhs_ref(), Ok(&Var(1)));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `App`lication.
-    pub fn lhs_ref(&self) -> Result<&Term, Error> {
-        if let Ok((lhs, _)) = self.unapp_ref() { Ok(lhs) } else { Err(NotAnApp) }
+    /// Returns a `TermError` if `self` is not an `App`lication.
+    pub fn lhs_ref(&self) -> Result<&Term, TermError> {
+        if let Ok((lhs, _)) = self.unapp_ref() { Ok(lhs) } else { Err(NotApp) }
     }
 
     /// Returns a mutable reference to the left-hand side term of an application.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(app(Var(1), Var(2)).lhs_mut(), Ok(&mut Var(1)));
     /// ```
-    pub fn lhs_mut(&mut self) -> Result<&mut Term, Error> {
-        if let Ok((lhs, _)) = self.unapp_mut() { Ok(lhs) } else { Err(NotAnApp) }
+    pub fn lhs_mut(&mut self) -> Result<&mut Term, TermError> {
+        if let Ok((lhs, _)) = self.unapp_mut() { Ok(lhs) } else { Err(NotApp) }
     }
 
     /// Returns the right-hand side term of an application. Consumes `self`.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(app(Var(1), Var(2)).rhs(), Ok(Var(2)));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `App`lication.
-    pub fn rhs(self) -> Result<Term, Error> {
-        if let Ok((_, rhs)) = self.unapp() { Ok(rhs) } else { Err(NotAnApp) }
+    /// Returns a `TermError` if `self` is not an `App`lication.
+    pub fn rhs(self) -> Result<Term, TermError> {
+        if let Ok((_, rhs)) = self.unapp() { Ok(rhs) } else { Err(NotApp) }
     }
 
     /// Returns a reference to the right-hand side term of an application.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(app(Var(1), Var(2)).rhs_ref(), Ok(&Var(2)));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `App`lication.
-    pub fn rhs_ref(&self) -> Result<&Term, Error> {
-        if let Ok((_, rhs)) = self.unapp_ref() { Ok(rhs) } else { Err(NotAnApp) }
+    /// Returns a `TermError` if `self` is not an `App`lication.
+    pub fn rhs_ref(&self) -> Result<&Term, TermError> {
+        if let Ok((_, rhs)) = self.unapp_ref() { Ok(rhs) } else { Err(NotApp) }
     }
 
     /// Returns a mutable reference to the right-hand side term of an application.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::*;
+    /// use lambda_calculus::*;
     ///
     /// assert_eq!(app(Var(1), Var(2)).rhs_mut(), Ok(&mut Var(2)));
     /// ```
     /// # Errors
     ///
-    /// The function will return an error if `self` is not an `App`lication.
-    pub fn rhs_mut(&mut self) -> Result<&mut Term, Error> {
-        if let Ok((_, rhs)) = self.unapp_mut() { Ok(rhs) } else { Err(NotAnApp) }
+    /// Returns a `TermError` if `self` is not an `App`lication.
+    pub fn rhs_mut(&mut self) -> Result<&mut Term, TermError> {
+        if let Ok((_, rhs)) = self.unapp_mut() { Ok(rhs) } else { Err(NotApp) }
     }
+
+    /// Returns `true` if `self` is a
+    /// [supercombinator](https://en.wikipedia.org/wiki/Supercombinator).
+    ///
+    /// # Example
+    /// ```
+    /// use lambda_calculus::*;
+    ///
+    /// let term1 = abs(app(Var(1), abs(Var(1)))); // λ 1 (λ 1)
+    /// let term2 = app(abs(Var(2)), abs(Var(1))); // (λ 2) (λ 1)
+    ///
+    /// assert_eq!(term1.is_supercombinator(), true);
+    /// assert_eq!(term2.is_supercombinator(), false);
+    /// ```
+    pub fn is_supercombinator(&self) -> bool {
+        let mut stack = Vec::new();
+        stack.push((0usize, self));
+        while let Some((depth, term)) = stack.pop() {
+            match *term {
+                Var(i) => if i > depth { return false },
+                Abs(ref t) => stack.push((depth + 1, t)),
+                App(ref f, ref a) => {
+                    stack.push((depth, f));
+                    stack.push((depth, a))
+                }
+            }
+        }
+        true
+    }
+
 }
 
 /// Wraps a `Term` in an `Abs`traction. Consumes its argument.
 ///
 /// # Example
 /// ```
-/// use lambda_calculus::term::*;
+/// use lambda_calculus::*;
 ///
 /// assert_eq!(abs(Var(1)), Abs(Box::new(Var(1))));
 /// ```
@@ -311,7 +330,7 @@ pub fn abs(term: Term) -> Term { Abs(Box::new(term)) }
 ///
 /// # Example
 /// ```
-/// use lambda_calculus::term::*;
+/// use lambda_calculus::*;
 ///
 /// assert_eq!(app(Var(1), Var(2)), App(Box::new(Var(1)), Box::new(Var(2))));
 /// ```
@@ -323,8 +342,11 @@ impl fmt::Display for Term {
     }
 }
 
-pub(crate) fn show_precedence_cla(term: &Term, context_precedence: usize, depth: u32) -> String {
+fn show_precedence_cla(term: &Term, context_precedence: usize, depth: u32) -> String {
     match *term {
+        Var(0) => {
+            format!("undefined")
+        },
         Var(i) => {
             if depth >= i as u32 {
                 format!("{}", from_u32(depth + 97 - i as u32).expect("error while printing term"))
@@ -358,8 +380,11 @@ impl fmt::Debug for Term {
     }
 }
 
-pub(crate) fn show_precedence_dbr(term: &Term, context_precedence: usize, depth: u32) -> String {
+fn show_precedence_dbr(term: &Term, context_precedence: usize, depth: u32) -> String {
     match *term {
+        Var(0) => {
+            format!("undefined")
+        },
         Var(i) => {
             format!("{:X}", i)
         },
@@ -393,7 +418,7 @@ fn parenthesize_if(input: &str, condition: bool) -> Cow<str> {
 /// # fn main() {
 /// use lambda_calculus::term::*;
 ///
-/// assert_eq!(app!(Var(1), Var(2), Var(3)), Var(1).app(Var(2)).app(Var(3)));
+/// assert_eq!(app!(Var(1), Var(2), Var(3)), app(app(Var(1), Var(2)), Var(3)));
 /// # }
 /// ```
 #[macro_export]
@@ -401,7 +426,7 @@ macro_rules! app {
     ($term1:expr, $($term2:expr),+) => {
         {
             let mut term = $term1;
-            $(term = term.app($term2);)*
+            $(term = app(term, $term2);)*
             term
         }
     };
@@ -440,7 +465,7 @@ mod tests {
     #[test]
     fn app_macro() {
         assert_eq!(app!(Var(4), app!(Var(1), Var(2), Var(3))),
-                   Var(4).app(Var(1).app(Var(2)).app(Var(3)))
+                   app(Var(4), app(app(Var(1), Var(2)), Var(3)))
         );
     }
 
@@ -482,4 +507,18 @@ mod tests {
         assert_eq!(&format!("{:?}", succ), "λλλ2(321)");
         assert_eq!(&format!("{:?}", pred), "λλλ3(λλ1(24))(λ2)(λ1)");
     }
+
+    #[test]
+    fn is_supercombinator() {
+        assert_eq!(abs(Var(1)).is_supercombinator(), true);
+        assert_eq!(app(abs(Var(1)), abs(Var(1))).is_supercombinator(), true);
+        assert_eq!(abs!(10, Var(10)).is_supercombinator(), true);
+        assert_eq!(abs!(10, app(Var(10), Var(10))).is_supercombinator(), true);
+
+        assert_eq!(Var(1).is_supercombinator(), false);
+        assert_eq!(abs(Var(2)).is_supercombinator(), false);
+        assert_eq!(app(abs(Var(1)), Var(1)).is_supercombinator(), false);
+        assert_eq!(abs!(10, Var(11)).is_supercombinator(), false);
+        assert_eq!(abs!(10, app(Var(10), Var(11))).is_supercombinator(), false);
+    }
 }
